package index_data

import (
	"encoding/json"
	"eta/eta_bridge/global"
	"eta/eta_bridge/models/pcsg"
	"fmt"
	"io/ioutil"
	"net/http"
)

// GetPCSGBloombergGeneralIndex 获取指标信息
func GetPCSGBloombergGeneralIndex(taskKey, frequency string) (indexes []pcsg.BaseFromBloombergApiIndexAndData, err error) {
	apiData, e := CurlPCSGBloombergGeneralDataApi(taskKey)
	if e != nil {
		err = fmt.Errorf("CurlPCSGBloombergDailyApi err: %s", e.Error())
		return
	}
	if len(apiData) == 0 {
		return
	}
	for _, v := range apiData {
		t := pcsg.FormatPythonBloombergGeneralData2Base(v, frequency)
		if t.IndexCode != "" {
			indexes = append(indexes, t)
		}
	}
	return
}

// CurlPCSGBloombergGeneralDataApi 请求通用格式数据接口
func CurlPCSGBloombergGeneralDataApi(taskKey string) (resultData []pcsg.PythonBloombergGeneralData, err error) {
	if taskKey == "" {
		err = fmt.Errorf("任务Key为空")
		return
	}
	if global.CONFIG.PCSG.BloombergApiUrl == "" {
		err = fmt.Errorf("服务地址为空")
		return
	}
	url := fmt.Sprintf("%s/api/bloomberg/general_data?task_key=%s", global.CONFIG.PCSG.BloombergApiUrl, taskKey)

	//resp, e := http.Post(url, "application/json", bytes.NewBuffer([]byte("")))
	resp, e := http.Get(url)
	if e != nil {
		err = fmt.Errorf("http post err: %s", e.Error())
		return
	}
	defer resp.Body.Close()

	b, e := ioutil.ReadAll(resp.Body)
	if e != nil {
		err = fmt.Errorf("resp body read err: %s", e.Error())
		return
	}
	if len(b) == 0 {
		err = fmt.Errorf("resp body is empty")
		return
	}

	result := new(pcsg.PythonBloombergGeneralResult)
	if e = json.Unmarshal(b, &result); e != nil {
		err = fmt.Errorf("result unmarshal err: %s\nresult: %s", e.Error(), string(b))
		return
	}
	if result.Code != 200 {
		err = fmt.Errorf("result: %s", string(b))
		return
	}
	resultData = result.Data
	return
}
